Jekyll-S3-TBC
==========================

Jekyll-S3-TBC is a static site generating framework implementing Twitter Bootstrap and CoffeeScript that focuses on web performance and makes it easy to deploy a site to be hosted on Amazon S3.  Jekyll-S3-TBC also uses Kramdown as the default markdown language to facilitate more control over content styling while enhancing speed.


## Install framework dependencies/prerequisites:


1. [Bundler](http://gembundler.com/)

    `gem install bundler`

2. ImageMagick and RMagick to automatically resize 2x images

  Mac:  Install [Homebrew](http://mxcl.github.com/homebrew/) then

  `brew install imagemagick`

  Linux: Use respective package manager and repos

  PC:  install [ImageMagick binary](http://www.imagemagick.org/script/binary-releases.php#windows)


3. Install optipng for optimizing PNGs:

  Mac: `brew install optipng`

  Linux: Use respective package manager and repos

  PC: [Install optipng windows release here](http://optipng.sourceforge.net/)


4. Install s3cmd to upload to Amazon S3: 

  Mac:  `brew install s3cmd`

  Linux: Use respective package manager and repos

  PC:  See [s3tools.org](http://s3tools.org/s3cmd) 

  All:

    `s3cmd --configure`

5.  Download [Google Closure](http://closure-compiler.googlecode.com/files/compiler-latest.zip), place jar file on your system.

  Mac (recommendation): `~/Library/Google/compiler-latest`

  PC/Linux: Place jar file on your system and update _build.thor in your project's folder with the location

6. Download [htmlcompressor](http://code.google.com/p/htmlcompressor/) jar, place in same directory as Closure jar.

7. In terminal window, change to project folder and run:

    `bundle install`

##  Configure Jekyll-S3-TBC

In _config.yml, add a URL value for `domain` (ex: http://ern.me). This is used for generating XML sitemaps.

In _build.thor change at least on of the following to deploy your site:

### To Deploy
  * `CDN_URL`: domain to prepend to static assets upon deploying - from your CDN provider
  * `BUCKET`: name of Amazon S3 bucket to deploy files to - any name your prefer - often your domain name
  * `SERVER_PROD`: if you aren't using Amazon S3, put address of server to rsync files to here
  * `SERVER_STAGING`: address of staging server if you have one
  * `WEB_DIR`: location of site files on your server

### Build class
  * `EXTERNAL_DIR`: you can integrate external dynamic applications that you may use elsewhere in your site. These applications can run through the Jekyll-S3-TBC stack to make sure templates in your dynamic apps get updated with references to CSS, JS, and image files that may be renamed by the versioning system. **If you do not have dynamic applications, you can blank this value out.**

## Build and deploy your site

Develop, test, build and deploy your site with the follow command line usage:

* Development mode, continually regenerates site files
  
    `thor dev`
  
* Build and serve on local machine

    `thor build`
  
* Build and deploy to production (be sure to set bucket name in _build.thor ex: `BUCKET = 'bucket_name'`)

    `thor deploy:production`
  
* Specify SSH login

    `thor deploy -l=[SSH USER]`
  
* List all commands available

    `thor list`
  
* Show help for one task

    `thor help [TASK]`


##  Additional Tags and Helpers

Include page specific meta and style tags in the front matter of individual pages

    ---content meta
    <meta name="viewport" content="width=device-width, initial-scale=1"/>
    ---content meta

    ---content styles
    <link rel="stylesheet" href="/stylesheets/special.css" type="text/css"/>
    ---content styles

Assign a page id in the front matter for individual pages to facilitate page linking helpers

    ----
    id: test 
    layout: default
    ----

  allows you to use 

    {% url test %}

  to link to this page.

Use absolute url to generate full url of a page

    {{ page.absolute_url }}


## Adding Content with Kramdown

[kramdown](http://kramdown.rubyforge.org/) is a pure Ruby library for parsing and converting a superset of Markdown.  Amoung other things, it allows content authors to add style tag to their content with a simple syntax variation on traditional Markdown.  Kramdown documents share the same .md extension and are automatically parsed as kramdown.  Given that kramdown is a Markdown superset, no learning curve is required because traditional Markdown just works.
  
    # This heading is special and centered.
    {: #special .center}

Renders...

    <h1 id="special" class="center">This heading is special</h1>


And easily mixing html and markdown in a markdown doc ...

    <div class="special-html" markdown="1">
    # This kramdown appears in an html block in a markdown (kramdown) document.
    </div>

Renders...

    <div class="special-html">

      <h1 class="special">This kramdown appears in an html block in a markdown (kramdown) document.</h1>

    </div>

See [kramdown quick reference](http://kramdown.rubyforge.org/quickref.html) for a short overview of kramdown syntax sugar or [kramdown Syntax](http://kramdown.rubyforge.org/quickref.html) for a full syntax documentation. 



# Known Issues

Jekyll-S3-TBC uses [Foreman](https://github.com/ddollar/foreman) to manage background process like Jekyll site regeneration, scss and coffeescript compilation and the Thin server.  Unfortunately there is a [Foreman Missing Output](https://github.com/ddollar/foreman/wiki/Missing-Output) problem that requires a workaround if you don't get all the feedback you would expect from running the processes in individual terminals.  This appears to be particularly true with the Jekyll regeneration notifications.  The work around is simple.  

Create `$HOME/.rubyopts.rb` with the following content:

    STDOUT.sync = true

In your .bash_profile or .zshrc add the following:

    export RUBYOUT="-r $HOME/.rubyopts.rb"


# Credits and License

## License

Copyright (c) 2012 David Franklin
Jekyll-S3-TBW is a derivative work based on [Ernie Bello's Jekyll-S3](https://github.com/ebello/Jekyll-S3). Jekyll-S3-TBW is free software and may be redistributed under the terms specified in the [LICENSE](https://github.com/dpfranklin/jekyll-s3-tbc/blob/master/LICENSE) and those specified by the licenses of its major components.

### Major components and their licenses:

* [CoffeeScript](http://coffeescript.org): [MIT](https://github.com/jashkenas/coffee-script/blob/master/LICENSE)
* [Compass](http://compass-style.org/): [slightly modified MIT License](https://github.com/chriseppstein/compass/blob/stable/LICENSE.markdown)
* [Foreman](https://github.com/ddollar/foreman): [MIT](http://www.opensource.org/licenses/mit-license.php)
* [Glyphicons Free](http://glyphicons.com/): [CC BY 3.0](http://creativecommons.org/licenses/by/3.0/)
* [Google Closure](https://developers.google.com/closure/compiler/): [Apache Licence v2.0](http://www.apache.org/licenses/LICENSE-2.0)
* [Google htmlcompressor](http://code.google.com/p/htmlcompressor/)[Apache Licence v2.0](http://www.apache.org/licenses/LICENSE-2.0)
* [ImageMagick](http://www.imagemagick.org): [ImageMagick Licence](http://imagemagick.mirrorcatalogs.com/www/license.html)
* [Jekyll](https://github.com/mojombo/jekyll): [MIT](https://github.com/mojombo/jekyll/blob/master/LICENSE)
* [Jekyll-S3](https://github.com/ebello/Jekyll-S3): [MIT](https://github.com/ebello/Jekyll-S3/blob/master/LICENSE)
* [jQuery](http://jquery.com/): [MIT](https://github.com/jquery/jquery/blob/master/MIT-LICENSE.txt)/[GPL license](https://github.com/jquery/jquery/blob/master/GPL-LICENSE.txt)
* [kramdown](http://kramdown.rubyforge.org/index.html): [GPL version 2](http://www.gnu.org/licenses/gpl-2.0.html)
* [Modernizr](http://modernizr.com/): [MIT/BSD license](http://modernizr.com/license/)
* [s3cmd](http://s3tools.org/s3cmd): [GPL version 2](http://www.gnu.org/licenses/gpl-2.0.html)
* [Thin web server](http://code.macournoyer.com/thin/): [Ruby License](http://www.ruby-lang.org/en/LICENSE.txt)
* [Thor](https://github.com/wycats/thor): [MIT](https://github.com/wycats/thor/blob/master/LICENSE.md)
* [Twitter Bootstrap](http://twitter.github.com/bootstrap/index.html): [Apache License v2.0](http://www.apache.org/licenses/LICENSE-2.0)
* [Uglifier](https://github.com/lautis/uglifier): [MIT](https://github.com/lautis/uglifier/blob/master/LICENSE.txt)








